#include "ipasir.h"

#include <assert.h>
#include <stdlib.h>
#include <stdio.h>

void die (const char * m) { fprintf (stderr, "error: %s\n", m); exit (1); } 

int main (int argc, char ** argv) {
  if (argc != 2) die ("usage: <dimacs>");
  FILE * file;
  if (!(file = fopen (argv[1], "r"))) die ("can not read file");
  printf ("c reading '%s'\n", argv[1]);
  int m, n; // maximum variable and number of clauses
  if (fscanf (file, "p cnf %d %d", &m, &n) != 2) die ("invalid header");
  printf ("c found 'p cnf %d %d' header\n", m, n);
  void * solver = ipasir_init (); // DONE initialize back-end 'solver'
  printf ("c %s back-end solver initialized\n",
    ipasir_signature ()); // DONE get and print name of back-end solver
  int c = 0, lit = 0;
  while (fscanf (file, "%d", &lit) == 1) {
    if (c == n) die ("too many clauses");
    ipasir_add (solver, lit);
    if (!lit) c++;
  }
  fclose (file);
  if (lit) die ("terminating zero missing");
  if (c != n) die ("clauses missing");
  int * model = malloc ((m + 1) * sizeof *model);
  long count = 0;
  for (;;) {
    int res = ipasir_solve (solver); // DONE call back-end solver
    if (res == 20) break;
    assert (res == 10);
    printf ("s SATISFIABLE\n");
    count++;
    printf ("v");
    int i;
    for (i = 1; i <= m; i++) { // save and print model
      model[i] = ipasir_val (solver, i); // DONE get value of variable 'i'
      printf ("%d ", model[i] < 0 ? -i : i);
    }
    printf ("0\n");
    fflush (stdout);
    // DONE add blocking clause to avoid the same model
    for (i = 1; i <= m; i++)
      ipasir_add (solver, model[i] < 0 ? i : -i);
    ipasir_add (solver, 0);
  }
  free (model);
  printf ("s SOLUTIONS %ld\n", count);
  return 0;
}
